@@ -31,7 +31,7 @@ typedef void (*rop3_with_pattern_handler_t)(pixman_image_t *d, pixman_image_t *s
 typedef void (*rop3_with_color_handler_t)(pixman_image_t *d, pixman_image_t *s,
                                           SpicePoint *src_pos, uint32_t rgb);
 
-typedef void (*rop3_test_handler_t)();
+typedef void (*rop3_test_handler_t)(void);
 
 #define ROP3_NUM_OPS 256
 
@@ -56,7 +56,7 @@ static void default_rop3_withe_color_handler(pixman_image_t *d, pixman_image_t *
     WARN("not implemented");
 }
 
-static void default_rop3_test_handler()
+static void default_rop3_test_handler(void)
 {
 }
 
@@ -127,7 +127,7 @@ static void rop3_handle_c##depth##_##name(pixman_image_t *d, pixman_image_t *s,
     }                                                                                           \
 }                                                                                               \
                                                                                                 \
-static void rop3_test##depth##_##name()                                                         \
+static void rop3_test##depth##_##name(void)                                                     \
 {                                                                                               \
     uint8_t d = 0xaa;                                                                           \
     uint8_t s = 0xcc;                                                                           \
