@@ -198,7 +198,7 @@ static void inputs_channel_release_msg_rcv_buf(RedChannelClient *rcc, SpiceDataH
      ((state & SPICE_MOUSE_BUTTON_MASK_MIDDLE) ? VD_AGENT_MBUTTON_MASK : 0) |    \
      ((state & SPICE_MOUSE_BUTTON_MASK_RIGHT) ? VD_AGENT_RBUTTON_MASK : 0))
 
-static void activate_modifiers_watch()
+static void activate_modifiers_watch(void)
 {
     core->timer_start(key_modifiers_timer, KEY_MODIFIERS_TTL);
 }
@@ -496,7 +496,9 @@ static void inputs_connect(RedChannel *channel, RedClient *client,
     icc = (InputsChannelClient*)red_channel_client_create(sizeof(InputsChannelClient),
                                                           channel,
                                                           client,
-                                                          stream);
+                                                          stream,
+                                                          num_common_caps, common_caps,
+                                                          num_caps, caps);
     icc->motion_count = 0;
     inputs_pipe_add_init(&icc->base);
 }
