@@ -19,6 +19,7 @@
 #include <config.h>
 #endif
 
+#include <inttypes.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <unistd.h>
@@ -39,17 +40,13 @@
 #include "server/demarshallers.h"
 #include "common/ring.h"
 #include "common/messages.h"
-#include "reds.h"
 #include "main_channel.h"
+#include "reds.h"
 #include "red_channel.h"
 #include "generated_marshallers.h"
 
 #define ZERO_BUF_SIZE 4096
 
-// approximate max receive message size for main channel
-#define RECEIVE_BUF_SIZE \
-    (4096 + (REDS_AGENT_WINDOW_SIZE + REDS_NUM_INTERNAL_AGENT_MESSAGES) * SPICE_AGENT_MAX_DATA_SIZE)
-
 #define REDS_MAX_SEND_IOVEC 100
 
 #define NET_TEST_WARMUP_BYTES 0
@@ -114,16 +111,6 @@ typedef struct NotifyPipeItem {
     int mess_len;
 } NotifyPipeItem;
 
-typedef struct MigrateBeginPipeItem {
-    PipeItem base;
-    int port;
-    int sport;
-    char *host;
-    uint16_t cert_pub_key_type;
-    uint32_t cert_pub_key_len;
-    uint8_t *cert_pub_key;
-} MigrateBeginPipeItem;
-
 typedef struct MultiMediaTimePipeItem {
     PipeItem base;
     int time;
@@ -141,11 +128,10 @@ struct MainChannelClient {
     SpiceTimer *ping_timer;
     int ping_interval;
 #endif
-};
-
-struct MainChannel {
-    RedChannel base;
-    uint8_t recv_buf[RECEIVE_BUF_SIZE];
+    int mig_wait_connect;
+    int mig_connect_ok;
+    int mig_wait_prev_complete;
+    int init_sent;
 };
 
 enum NetTestStage {
@@ -155,6 +141,9 @@ enum NetTestStage {
     NET_TEST_STAGE_RATE,
 };
 
+static void main_channel_release_pipe_item(RedChannelClient *rcc,
+                                           PipeItem *base, int item_pushed);
+
 int main_channel_is_connected(MainChannel *main_chan)
 {
     return red_channel_is_connected(&main_chan->base);
@@ -224,13 +213,18 @@ static PipeItem *main_ping_item_new(MainChannelClient *mcc, int size)
     return &item->base;
 }
 
+typedef struct MainTokensItemInfo {
+    uint32_t num_tokens;
+} MainTokensItemInfo;
+
 static PipeItem *main_tokens_item_new(RedChannelClient *rcc, void *data, int num)
 {
     TokensPipeItem *item = spice_malloc(sizeof(TokensPipeItem));
+    MainTokensItemInfo *init = data;
 
     red_channel_pipe_item_init(rcc->channel, &item->base,
                                SPICE_MSG_MAIN_AGENT_TOKEN);
-    item->tokens = (uint64_t)data;
+    item->tokens = init->num_tokens;
     return &item->base;
 }
 
@@ -292,33 +286,6 @@ static PipeItem *main_notify_item_new(RedChannelClient *rcc, void *data, int num
     return &item->base;
 }
 
-typedef struct MigrateBeginItemInfo {
-    int port;
-    int sport;
-    char *host;
-    uint16_t cert_pub_key_type;
-    uint32_t cert_pub_key_len;
-    uint8_t *cert_pub_key;
-} MigrateBeginItemInfo;
-
-// TODO: MC: migration is not tested at all with multiclient.
-static PipeItem *main_migrate_begin_item_new(
-    RedChannelClient *rcc, void *data, int num)
-{
-    MigrateBeginPipeItem *item = spice_malloc(sizeof(MigrateBeginPipeItem));
-    MigrateBeginItemInfo *info = data;
-
-    red_channel_pipe_item_init(rcc->channel, &item->base,
-                               SPICE_MSG_MAIN_MIGRATE_BEGIN);
-    item->port = info->port;
-    item->sport = info->sport;
-    item->host = info->host;
-    item->cert_pub_key_type = info->cert_pub_key_type;
-    item->cert_pub_key_len = info->cert_pub_key_len;
-    item->cert_pub_key = info->cert_pub_key;
-    return &item->base;
-}
-
 static PipeItem *main_multi_media_time_item_new(
     RedChannelClient *rcc, void *data, int num)
 {
@@ -333,6 +300,11 @@ static PipeItem *main_multi_media_time_item_new(
 
 static void main_channel_push_channels(MainChannelClient *mcc)
 {
+    if (red_client_during_migrate_at_target(mcc->base.client)) {
+        red_printf("warning: ignoring unexpected SPICE_MSGC_MAIN_ATTACH_CHANNELS"
+                   "during migration");
+        return;
+    }
     red_channel_client_pipe_add_type(&mcc->base, SPICE_MSG_MAIN_CHANNELS_LIST);
 }
 
@@ -420,8 +392,10 @@ static void main_channel_marshall_agent_disconnected(SpiceMarshaller *m)
 // TODO: make this targeted (requires change to agent token accounting)
 void main_channel_push_tokens(MainChannel *main_chan, uint32_t num_tokens)
 {
+    MainTokensItemInfo init = {.num_tokens = num_tokens};
+
     red_channel_pipes_new_add_push(&main_chan->base,
-        main_tokens_item_new, (void*)(uint64_t)num_tokens);
+        main_tokens_item_new, &init);
 }
 
 static void main_channel_marshall_tokens(SpiceMarshaller *m, uint32_t num_tokens)
@@ -495,7 +469,7 @@ static uint64_t main_channel_handle_migrate_data(RedChannelClient *base,
     return TRUE;
 }
 
-void main_channel_push_init(MainChannelClient *mcc, int connection_id,
+void main_channel_push_init(MainChannelClient *mcc,
     int display_channels_hint, int current_mouse_mode,
     int is_client_mouse_allowed, int multi_media_time,
     int ram_hint)
@@ -503,7 +477,7 @@ void main_channel_push_init(MainChannelClient *mcc, int connection_id,
     PipeItem *item;
 
     item = main_init_item_new(mcc,
-             connection_id, display_channels_hint, current_mouse_mode,
+             mcc->connection_id, display_channels_hint, current_mouse_mode,
              is_client_mouse_allowed, multi_media_time, ram_hint);
     red_channel_client_pipe_add_push(&mcc->base, item);
 }
@@ -559,35 +533,23 @@ static void main_channel_marshall_notify(SpiceMarshaller *m, NotifyPipeItem *ite
     spice_marshaller_add(m, item->mess, item->mess_len + 1);
 }
 
-void main_channel_push_migrate_begin(MainChannel *main_chan, int port, int sport,
-    char *host, uint16_t cert_pub_key_type, uint32_t cert_pub_key_len,
-    uint8_t *cert_pub_key)
-{
-    MigrateBeginItemInfo info = {
-        .port =port,
-        .sport = sport,
-        .host = host,
-        .cert_pub_key_type = cert_pub_key_type,
-        .cert_pub_key_len = cert_pub_key_len,
-        .cert_pub_key = cert_pub_key,
-    };
-
-    red_channel_pipes_new_add_push(&main_chan->base,
-        main_migrate_begin_item_new, &info);
-}
-
-static void main_channel_marshall_migrate_begin(SpiceMarshaller *m,
-    MigrateBeginPipeItem *item)
+static void main_channel_marshall_migrate_begin(SpiceMarshaller *m, RedChannelClient *rcc)
 {
     SpiceMsgMainMigrationBegin migrate;
+    MainChannel *main_ch;
 
-    migrate.port = item->port;
-    migrate.sport = item->sport;
-    migrate.host_size = strlen(item->host) + 1;
-    migrate.host_data = (uint8_t *)item->host;
-    migrate.pub_key_type = item->cert_pub_key_type;
-    migrate.pub_key_size = item->cert_pub_key_len;
-    migrate.pub_key_data = item->cert_pub_key;
+    main_ch = SPICE_CONTAINEROF(rcc->channel, MainChannel, base);
+    migrate.port = main_ch->mig_target.port;
+    migrate.sport = main_ch->mig_target.sport;
+    migrate.host_size = strlen(main_ch->mig_target.host) + 1;
+    migrate.host_data = (uint8_t *)main_ch->mig_target.host;
+    if (main_ch->mig_target.cert_subject) {
+        migrate.cert_subject_size = strlen(main_ch->mig_target.cert_subject) + 1;
+        migrate.cert_subject_data = (uint8_t *)main_ch->mig_target.cert_subject;
+    } else {
+        migrate.cert_subject_size = 0;
+        migrate.cert_subject_data = NULL;
+    }
     spice_marshall_msg_main_migrate_begin(m, &migrate);
 }
 
@@ -604,11 +566,6 @@ static void main_channel_marshall_migrate(SpiceMarshaller *m)
     spice_marshall_msg_migrate(m, &migrate);
 }
 
-void main_channel_push_migrate_cancel(MainChannel *main_chan)
-{
-    red_channel_pipes_add_type(&main_chan->base, SPICE_MSG_MAIN_MIGRATE_CANCEL);
-}
-
 void main_channel_push_multi_media_time(MainChannel *main_chan, int time)
 {
     MultiMediaTimePipeItem info = {
@@ -619,33 +576,43 @@ void main_channel_push_multi_media_time(MainChannel *main_chan, int time)
         main_multi_media_time_item_new, &info);
 }
 
-static PipeItem *main_migrate_switch_item_new(RedChannelClient *rcc,
-                                              void *data, int num)
+static void main_channel_fill_mig_target(MainChannel *main_channel, RedsMigSpice *mig_target)
 {
-    RefsPipeItem *item = spice_malloc(sizeof(*item));
-
-    item->refs = data;
-    red_channel_pipe_item_init(rcc->channel, &item->base,
-                               SPICE_MSG_MAIN_MIGRATE_SWITCH_HOST);
-    return &item->base;
+    ASSERT(mig_target);
+    free(main_channel->mig_target.host);
+    main_channel->mig_target.host = strdup(mig_target->host);
+    free(main_channel->mig_target.cert_subject);
+    if (mig_target->cert_subject) {
+        main_channel->mig_target.cert_subject = strdup(mig_target->cert_subject);
+    }
+    main_channel->mig_target.port = mig_target->port;
+    main_channel->mig_target.sport = mig_target->sport;
 }
 
-void main_channel_push_migrate_switch(MainChannel *main_chan)
+void main_channel_migrate_switch(MainChannel *main_chan, RedsMigSpice *mig_target)
 {
-    int *refs = spice_malloc0(sizeof(int));
-
-    *refs = main_chan->base.clients_num;
-    red_channel_pipes_new_add_push(&main_chan->base,
-        main_migrate_switch_item_new, (void*)refs);
+    main_channel_fill_mig_target(main_chan, mig_target);
+    red_channel_pipes_add_type(&main_chan->base, SPICE_MSG_MAIN_MIGRATE_SWITCH_HOST);
 }
 
-static void main_channel_marshall_migrate_switch(SpiceMarshaller *m)
+static void main_channel_marshall_migrate_switch(SpiceMarshaller *m, RedChannelClient *rcc)
 {
     SpiceMsgMainMigrationSwitchHost migrate;
+    MainChannel *main_ch;
 
     red_printf("");
-
-    reds_fill_mig_switch(&migrate);
+    main_ch = SPICE_CONTAINEROF(rcc->channel, MainChannel, base);
+    migrate.port = main_ch->mig_target.port;
+    migrate.sport = main_ch->mig_target.sport;
+    migrate.host_size = strlen(main_ch->mig_target.host) + 1;
+    migrate.host_data = (uint8_t *)main_ch->mig_target.host;
+    if (main_ch->mig_target.cert_subject) {
+        migrate.cert_subject_size = strlen(main_ch->mig_target.cert_subject) + 1;
+        migrate.cert_subject_data = (uint8_t *)main_ch->mig_target.cert_subject;
+    } else {
+        migrate.cert_subject_size = 0;
+        migrate.cert_subject_data = NULL;
+    }
     spice_marshall_msg_main_migrate_switch_host(m, &migrate);
 }
 
@@ -663,6 +630,13 @@ static void main_channel_send_item(RedChannelClient *rcc, PipeItem *base)
     MainChannelClient *mcc = SPICE_CONTAINEROF(rcc, MainChannelClient, base);
     SpiceMarshaller *m = red_channel_client_get_marshaller(rcc);
 
+    if (!mcc->init_sent && base->type != SPICE_MSG_MAIN_INIT) {
+        red_printf("Init msg for client %p was not sent yet "
+                   "(client is probably during migration). Ignoring msg type %d",
+                   rcc->client, base->type);
+        main_channel_release_pipe_item(rcc, base, FALSE);
+        return;
+    }
     red_channel_client_init_send_data(rcc, base->type, base);
     switch (base->type) {
         case SPICE_MSG_MAIN_CHANNELS_LIST:
@@ -697,6 +671,7 @@ static void main_channel_send_item(RedChannelClient *rcc, PipeItem *base)
                 mcc->ping_id);
             break;
         case SPICE_MSG_MAIN_INIT:
+            mcc->init_sent = TRUE;
             main_channel_marshall_init(m,
                 SPICE_CONTAINEROF(base, InitPipeItem, base));
             break;
@@ -708,15 +683,14 @@ static void main_channel_send_item(RedChannelClient *rcc, PipeItem *base)
             main_channel_marshall_migrate(m);
             break;
         case SPICE_MSG_MAIN_MIGRATE_BEGIN:
-            main_channel_marshall_migrate_begin(m,
-                SPICE_CONTAINEROF(base, MigrateBeginPipeItem, base));
+            main_channel_marshall_migrate_begin(m, rcc);
             break;
         case SPICE_MSG_MAIN_MULTI_MEDIA_TIME:
             main_channel_marshall_multi_media_time(m,
                 SPICE_CONTAINEROF(base, MultiMediaTimePipeItem, base));
             break;
         case SPICE_MSG_MAIN_MIGRATE_SWITCH_HOST:
-            main_channel_marshall_migrate_switch(m);
+            main_channel_marshall_migrate_switch(m, rcc);
             break;
     };
     red_channel_client_begin_send_message(rcc);
@@ -729,27 +703,58 @@ static void main_channel_release_pipe_item(RedChannelClient *rcc,
         case SPICE_MSG_MAIN_AGENT_DATA: {
             AgentDataPipeItem *data = (AgentDataPipeItem*)base;
             if (!--data->refs->refs) {
-                red_printf_debug(1, "SPICE_MSG_MAIN_AGENT_DATA %p %p, %d",
+                red_printf_debug(1, "MAIN", "SPICE_MSG_MAIN_AGENT_DATA %p %p, %d",
                                  data, data->refs, data->refs->refs);
                 free(data->refs);
                 data->free_data(data->data, data->opaque);
             }
             break;
         }
-        case SPICE_MSG_MAIN_MIGRATE_SWITCH_HOST: {
-            RefsPipeItem *data = (RefsPipeItem*)base;
-            if (!--*(data->refs)) {
-                free(data->refs);
-                reds_mig_release();
-            }
-            break;
-        }
         default:
             break;
     }
     free(base);
 }
 
+void main_channel_client_handle_migrate_connected(MainChannelClient *mcc, int success)
+{
+    red_printf("client %p connected: %d", mcc->base.client, success);
+    if (mcc->mig_wait_connect) {
+        MainChannel *main_channel = SPICE_CONTAINEROF(mcc->base.channel, MainChannel, base);
+
+        mcc->mig_wait_connect = FALSE;
+        mcc->mig_connect_ok = success;
+        ASSERT(main_channel->num_clients_mig_wait);
+        if (!--main_channel->num_clients_mig_wait) {
+            reds_on_main_migrate_connected();
+        }
+    } else {
+        if (success) {
+            red_printf("client %p MIGRATE_CANCEL", mcc->base.client);
+            red_channel_client_pipe_add_type(&mcc->base, SPICE_MSG_MAIN_MIGRATE_CANCEL);
+        }
+    }
+}
+
+void main_channel_client_handle_migrate_end(MainChannelClient *mcc)
+{
+    if (!red_client_during_migrate_at_target(mcc->base.client)) {
+        red_printf("unexpected SPICE_MSGC_MIGRATE_END");
+        return;
+    }
+    if (!red_channel_client_test_remote_cap(&mcc->base,
+                                            SPICE_MAIN_CAP_SEMI_SEAMLESS_MIGRATE)) {
+        red_printf("unexpected SPICE_MSGC_MIGRATE_END, "
+                   "client does not support semi-seamless migration");
+            return;
+    }
+    red_client_migrate_complete(mcc->base.client);
+    if (mcc->mig_wait_prev_complete) {
+        red_channel_client_pipe_add_type(&mcc->base, SPICE_MSG_MAIN_MIGRATE_BEGIN);
+        mcc->mig_wait_connect = TRUE;
+        mcc->mig_wait_prev_complete = FALSE;
+    }
+}
 static int main_channel_handle_parsed(RedChannelClient *rcc, uint32_t size, uint16_t type, void *message)
 {
     MainChannel *main_chan = SPICE_CONTAINEROF(rcc->channel, MainChannel, base);
@@ -773,12 +778,10 @@ static int main_channel_handle_parsed(RedChannelClient *rcc, uint32_t size, uint
         main_channel_push_channels(mcc);
         break;
     case SPICE_MSGC_MAIN_MIGRATE_CONNECTED:
-        red_printf("connected");
-        reds_on_main_migrate_connected();
+        main_channel_client_handle_migrate_connected(mcc, TRUE);
         break;
     case SPICE_MSGC_MAIN_MIGRATE_CONNECT_ERROR:
-        red_printf("mig connect error");
-        reds_on_main_migrate_connect_error();
+        main_channel_client_handle_migrate_connected(mcc, FALSE);
         break;
     case SPICE_MSGC_MAIN_MOUSE_MODE_REQUEST:
         reds_on_main_mouse_mode_request(message, size);
@@ -807,13 +810,14 @@ static int main_channel_handle_parsed(RedChannelClient *rcc, uint32_t size, uint
                 if (roundtrip <= mcc->latency) {
                     // probably high load on client or server result with incorrect values
                     mcc->latency = 0;
-                    red_printf("net test: invalid values, latency %lu roundtrip %lu. assuming high"
+                    red_printf("net test: invalid values, latency %" PRIu64
+                               " roundtrip %" PRIu64 ". assuming high"
                                "bandwidth", mcc->latency, roundtrip);
                     break;
                 }
                 mcc->bitrate_per_sec = (uint64_t)(NET_TEST_BYTES * 8) * 1000000
                                         / (roundtrip - mcc->latency);
-                red_printf("net test: latency %f ms, bitrate %lu bps (%f Mbps)%s",
+                red_printf("net test: latency %f ms, bitrate %"PRIu64" bps (%f Mbps)%s",
                            (double)mcc->latency / 1000,
                            mcc->bitrate_per_sec,
                            (double)mcc->bitrate_per_sec / 1024 / 1024,
@@ -839,6 +843,9 @@ static int main_channel_handle_parsed(RedChannelClient *rcc, uint32_t size, uint
                 }
     case SPICE_MSGC_DISCONNECTING:
         break;
+    case SPICE_MSGC_MAIN_MIGRATE_END:
+        main_channel_client_handle_migrate_end(mcc);
+        break;
     default:
         red_printf("unexpected type %d", type);
     }
@@ -911,12 +918,18 @@ uint32_t main_channel_client_get_link_id(MainChannelClient *mcc)
     return mcc->connection_id;
 }
 
-MainChannelClient *main_channel_client_create(MainChannel *main_chan, RedClient *client,
-                                              RedsStream *stream, uint32_t connection_id)
+static MainChannelClient *main_channel_client_create(MainChannel *main_chan, RedClient *client,
+                                                     RedsStream *stream, uint32_t connection_id,
+                                                     int num_common_caps, uint32_t *common_caps,
+                                                     int num_caps, uint32_t *caps)
 {
     MainChannelClient *mcc = (MainChannelClient*)red_channel_client_create(sizeof(MainChannelClient),
                                                                            &main_chan->base,
-                                                                           client, stream);
+                                                                           client, stream,
+                                                                           num_common_caps,
+                                                                           common_caps,
+                                                                           num_caps,
+                                                                           caps);
 
     mcc->connection_id = connection_id;
     mcc->bitrate_per_sec = ~0;
@@ -942,7 +955,9 @@ MainChannelClient *main_channel_link(MainChannel *channel, RedClient *client,
     // into usage somewhere (not an issue until we return migration to it's
     // former glory)
     red_printf("add main channel client");
-    mcc = main_channel_client_create(channel, client, stream, connection_id);
+    mcc = main_channel_client_create(channel, client, stream, connection_id,
+                                     num_common_caps, common_caps,
+                                     num_caps, caps);
     return mcc;
 }
 
@@ -1002,6 +1017,96 @@ MainChannel* main_channel_init(void)
                                         main_channel_handle_parsed,
                                         &channel_cbs);
     ASSERT(channel);
-
+    red_channel_set_cap(channel, SPICE_MAIN_CAP_SEMI_SEAMLESS_MIGRATE);
     return (MainChannel *)channel;
 }
+
+RedChannelClient* main_channel_client_get_base(MainChannelClient* mcc)
+{
+    ASSERT(mcc);
+    return &mcc->base;
+}
+
+int main_channel_migrate_connect(MainChannel *main_channel, RedsMigSpice *mig_target)
+{
+    RingItem *client_link;
+
+    main_channel_fill_mig_target(main_channel, mig_target);
+    main_channel->num_clients_mig_wait = 0;
+
+    RING_FOREACH(client_link, &main_channel->base.clients) {
+        MainChannelClient * mcc = SPICE_CONTAINEROF(client_link, MainChannelClient, base.channel_link);
+        if (red_channel_client_test_remote_cap(&mcc->base,
+                                               SPICE_MAIN_CAP_SEMI_SEAMLESS_MIGRATE)) {
+            if (red_client_during_migrate_at_target(mcc->base.client)) {
+                red_printf("client %p: wait till previous migration completes", mcc->base.client);
+                mcc->mig_wait_prev_complete = TRUE;
+            } else {
+                red_channel_client_pipe_add_type(&mcc->base, SPICE_MSG_MAIN_MIGRATE_BEGIN);
+                mcc->mig_wait_connect = TRUE;
+            }
+            mcc->mig_connect_ok = FALSE;
+            main_channel->num_clients_mig_wait++;
+        }
+    }
+    return main_channel->num_clients_mig_wait;
+}
+
+void main_channel_migrate_cancel_wait(MainChannel *main_chan)
+{
+    RingItem *client_link;
+
+    RING_FOREACH(client_link, &main_chan->base.clients) {
+        MainChannelClient *mcc;
+
+        mcc = SPICE_CONTAINEROF(client_link, MainChannelClient, base.channel_link);
+        if (mcc->mig_wait_connect) {
+            red_printf("client %p cancel wait connect", mcc->base.client);
+            mcc->mig_wait_connect = FALSE;
+            mcc->mig_connect_ok = FALSE;
+        }
+        mcc->mig_wait_prev_complete = FALSE;
+    }
+    main_chan->num_clients_mig_wait = 0;
+}
+
+int main_channel_migrate_complete(MainChannel *main_chan, int success)
+{
+    RingItem *client_link;
+    int semi_seamless_count = 0;
+
+    red_printf("");
+
+    if (ring_is_empty(&main_chan->base.clients)) {
+        red_printf("no peer connected");
+        return 0;
+    }
+
+    RING_FOREACH(client_link, &main_chan->base.clients) {
+        MainChannelClient *mcc;
+        int semi_seamless_support;
+
+        mcc = SPICE_CONTAINEROF(client_link, MainChannelClient, base.channel_link);
+        semi_seamless_support = red_channel_client_test_remote_cap(&mcc->base,
+                                                   SPICE_MAIN_CAP_SEMI_SEAMLESS_MIGRATE);
+        if (semi_seamless_support && mcc->mig_connect_ok) {
+            if (success) {
+                red_printf("client %p MIGRATE_END", mcc->base.client);
+                red_channel_client_pipe_add_type(&mcc->base, SPICE_MSG_MAIN_MIGRATE_END);
+                semi_seamless_count++;
+            } else {
+                red_printf("client %p MIGRATE_CANCEL", mcc->base.client);
+                red_channel_client_pipe_add_type(&mcc->base, SPICE_MSG_MAIN_MIGRATE_CANCEL);
+            }
+        } else {
+            if (success) {
+                red_printf("client %p SWITCH_HOST", mcc->base.client);
+                red_channel_client_pipe_add_type(&mcc->base, SPICE_MSG_MAIN_MIGRATE_SWITCH_HOST);
+            }
+        }
+        mcc->mig_connect_ok = FALSE;
+        mcc->mig_wait_connect = FALSE;
+   }
+   return semi_seamless_count;
+}
+
