@@ -26,10 +26,12 @@ typedef struct MJpegEncoder MJpegEncoder;
 MJpegEncoder *mjpeg_encoder_new(int width, int height);
 void mjpeg_encoder_destroy(MJpegEncoder *encoder);
 
-uint8_t *mjpeg_encoder_get_frame(MJpegEncoder *encoder);
-size_t mjpeg_encoder_get_frame_stride(MJpegEncoder *encoder);
-int mjpeg_encoder_encode_frame(MJpegEncoder *encoder,
-                               uint8_t *buffer, size_t buffer_len);
+uint8_t mjpeg_encoder_get_bytes_per_pixel(MJpegEncoder *encoder);
+int mjpeg_encoder_start_frame(MJpegEncoder *encoder, SpiceBitmapFmt format,
+                              uint8_t **dest, size_t *dest_len);
+int mjpeg_encoder_encode_scanline(MJpegEncoder *encoder, uint8_t *src_pixels,
+                                  size_t image_width);
+size_t mjpeg_encoder_end_frame(MJpegEncoder *encoder);
 
 
 #endif
