@@ -95,6 +95,17 @@ struct SpiceNetWireState {
     struct TunnelWorker *worker;
 };
 
+struct SpiceMigrateState {
+    int dummy;
+};
+
+typedef struct RedsMigSpice {
+    char *host;
+    char *cert_subject;
+    int port;
+    int sport;
+} RedsMigSpice;
+
 ssize_t reds_stream_read(RedsStream *s, void *buf, size_t nbyte);
 ssize_t reds_stream_write(RedsStream *s, const void *buf, size_t nbyte);
 ssize_t reds_stream_writev(RedsStream *s, const struct iovec *iov, int iovcnt);
@@ -122,21 +133,20 @@ void reds_client_disconnect(RedClient *client);
 typedef struct MainMigrateData MainMigrateData;
 void reds_marshall_migrate_data_item(SpiceMarshaller *m, MainMigrateData *data);
 void reds_fill_channels(SpiceMsgChannels *channels_info);
-void reds_fill_mig_switch(SpiceMsgMainMigrationSwitchHost *migrate);
-void reds_mig_release(void);
 int reds_num_of_channels(void);
 int reds_num_of_clients(void);
 #ifdef RED_STATISTICS
 void reds_update_stat_value(uint32_t value);
 #endif
 
-// callbacks from main channel messages
+/* callbacks from main channel messages */
+
 void reds_on_main_agent_start(void);
 void reds_on_main_agent_data(MainChannelClient *mcc, void *message, size_t size);
-void reds_on_main_migrate_connected();
-void reds_on_main_migrate_connect_error();
+void reds_on_main_migrate_connected(void); //should be called when all the clients
+                                           // are connected to the target
 void reds_on_main_receive_migrate_data(MainMigrateData *data, uint8_t *end);
 void reds_on_main_mouse_mode_request(void *message, size_t size);
-
+void reds_on_client_migrate_complete(RedClient *client);
 #endif
 
