@@ -14,7 +14,7 @@ int simple_commands[] = {
 SpiceCoreInterface *core;
 SpiceServer *server;
 
-int main()
+int main(void)
 {
     core = basic_event_loop_init();
     server = test_init(core);
